#####################################################################
# Copyright (C) 2000 Artur Trzewik
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
######################################################################
# Header
# CVS - $Id: schemaedit.tcl,v 1.8 2000/10/10 10:16:27 artur Exp $
#
# Information:
# Modul (benutzt von Schema Editor)
# Hier eine Sammlung von Prozeduren fr das Editieren von
# DB Schema
#
# Aus effizientz Grnden knnen die Objekte nur eimal initialisiert werden
# und gezeichnet mit map und nacher fr verschieden Nodes verwendet werden
# dazwische knnen sie mit forget aus der Oberflcher verschwinden
# mit proc handleNewNode $node
#
# Die Eigenschaftenfenster werden mit allen InputWidget sofort
# kreiert aber nur die ntige werden angezeigt. Dafr wurde FrameContainer
# um Prozeduren hide und show erweitert.

Class SchemaEdit -parameter {
    {domNode {}}
}
# Die Abgeleitete Klassen sollten in metadata proc und proc spec beschreiben
# Welche Eigenschaften sie haben
#
# framelist ist eine keydlist (tclx) mit Aufbau
# (name FrameObjekt)
# gedacht fr setzen und auslesen von Argumenten
#
SchemaEdit instproc map window {
    [self] instvar  win framelist inputframe
    global lan
    # Hier werden die Eigenschaft Listen Vorbereitet
    set propspec [[[self] info class] metadata propspec]
    foreach prop [[[self] info  class] metadata prop] {
	# Das zweite sollte die bersetztung werden
	set cardattr [list [lindex $prop 0] [lindex $prop 0]] 
	set alist {}
	foreach attr [lindex $prop 1] {
	    set klist [keylget propspec $attr]
	    # Label sprachspezifisch setzten
	    keylset klist label [set lan(label.$attr)]
	    # Name setzen genau so wie die Attribut Name
	    keylset klist name $attr
	    lappend alist $klist
	}
	lappend inputspec [list $cardattr $alist]	
    }
    set win $window
    if {![info exist inputspec]} { error "Internal Error: keine Specifikationen fr Edit"}
    if {[llength $inputspec]==1} {
	# Hier wenn nur ein Optionsblatt also kein Notebook
	set inputframe [self]::inputframeobj
	WidgetFactory make_inputs_keylist $inputframe [lindex [lindex $inputspec 0] 1]
	lappend framelist [list [lindex [lindex [lindex $inputspec 0] 0] 0] $inputframe]
    } else {
	set inputframe [NoteBookContainer [self]::notebook]
	foreach card $inputspec {
	    set cardattr [lindex $card 0]
	    set input [lindex $card 1]
	    set inputname ${inputframe}::[Object autoname bookcard]
	    WidgetFactory make_inputs_keylist $inputname $input
	    lappend framelist [list [lindex $cardattr 0] $inputname]
	    $inputframe addcard $inputname $cardattr
	    set card_arr([lindex $cardattr 0]) $inputname
	}
    }
    $inputframe map $win
    #puts "inputframe $inputframe"
}
# Eine Standart Prozedur nimmt an es sind alle Eigenschaften
# als Attributten gespeichert
SchemaEdit instproc handleNewNode node {
    [self] instvar domNode framelist
    if {$domNode==$node} {return}
    set domNode $node
    foreach prop [[[self] info class] metadata prop] {
	set pframe [keylget framelist [lindex $prop 0]]
	set klist {}
 	foreach attr [lindex $prop 1] {
	    if {[$domNode hasAttribute $attr]} {
		keylset klist $attr [$domNode getAttribute $attr]
	    } else {
		keylset klist $attr {}
	    }
 	}
	$pframe set_defaults $klist
    }
}
SchemaEdit instproc unmap {} {
    [self] instvar win
    destroy $win
}
#
# Zentrale prozeduren fr Operationen (move, add, delete) der tDom Noden
# Die Prozeduren knnen per mixins angezappft werden und dir Opertationen
# etwa auf gui bergehen
#
# Jetzt Allerdings werden die Operationen direkt zu ProgGui Objekt
# weitergegeben
SchemaEdit instproc addElem {node posnode {append 1}} {
    if {$append} {
	$posnode appendChild $node
    } else {
	[$posnode parentNode] insertBefore $node $posnode
    }
    ProgGui addElem $node $posnode $append
}
SchemaEdit instproc moveElem {node posnode {append 1}} {
    [$node parentNode] removeChild $node
    if {$append} {
	$posnode appendChild $node
    } else {
	[$posnode parentNode] insertBefore $node $posnode
    }
    ProgGui moveElem $node $posnode $append
}
SchemaEdit instproc deleteElem {node} {
    [$node parentNode] removeChild $node
    ProgGui deleteElem $node
}
SchemaEdit instproc newName {node {name {}}} {
    if {$name==""} {
	ProgGui newName $node [$node getAttribute name]
    } else {
	$node setAttribute name $name
	ProgGui newName $node $name
    }
}
# bertreg die nderung des Node Types auf GUI 
# (attribut->primary_key, table->type
SchemaEdit instproc showChanges {node} {
    #puts "check $node"
    ProgGui applyChanges $node
}

#
# Diese Prozedur Sammelt alle Vernderungen fr die Einzelnen Eigenschaften
# und gibt sie weiter der vererbten Klasse zur verarbeiteung
# 
SchemaEdit instproc applyChanges {} {
    [self] instvar inputframe
    $inputframe update_values
    set changes [$inputframe get_changes]
    #puts "Changes $changes"
    if {$changes!=""} {
	[self] modifyProp $changes
    }
}
# Eine Standart Prozedur, geht davon aus, da alle Eigenschaften als 
# Tagattribute geschrieben werden
SchemaEdit instproc modifyProp {changes} {
    [self] instvar domNode
    foreach key [keylkeys changes] {
	$domNode setAttribute $key [keylget changes $key]
    }
}
# Specializierung Editieren von Database Eigenschaften
Class SchemaEditDatabase -superclass SchemaEdit
SchemaEditDatabase metadata add prop
SchemaEditDatabase metadata prop {
    {primary {name type}}
}
SchemaEditDatabase metadata add propspec
SchemaEditDatabase metadata propspec {
    {name {{type OLabelEntry} {textwidth 20} {state disabled}}}
    {type {{type OLabelEntry} {textwidth 20} {state disabled}}}
}
SchemaEditDatabase instproc map win {
    next
} {
    [$domNode nodeType]=="ELEMENT_NODE" && [$domNode nodeName]=="table"
} {{#post}}
SchemaEditDatabase instproc handleNewNode node {
    next
}

# Specializierung Editieren von Table Eigenschaften
Class SchemaEditTable -superclass SchemaEdit
SchemaEditTable metadata add prop
SchemaEditTable metadata prop {
    {primary {name type description}}
}
SchemaEditTable metadata add propspec
SchemaEditTable metadata propspec {
    {name {{type OLabelEntry} {textwidth 20} {state disabled}}}
    {type {{type OLabelEntry} {textwidth 20} {state disabled}}}
    {description {{type OLabelText} }}
}
SchemaEditTable instproc map win {
    next
} {
    [$domNode nodeType]=="ELEMENT_NODE" && [$domNode nodeName]=="table"
} {{#post}}
SchemaEditTable instproc handleNewNode node {
    next
}
SchemaEditTable instproc modifyProp {changes} {
    [self] instvar domNode
    if {[lcontain [keylkeys changes] description]} {
	tdom_setpername $domNode description [keylget changes description]
    }
    foreach key [keylkeys changes] {
	if {$key=="description"} continue
	$domNode setAttribute $key [keylget changes $key]
    }
}
SchemaEditTable instproc handleNewNode node {
    [self] instvar domNode framelist
    if {$domNode==$node} {return}
    set domNode $node
    foreach prop [[[self] info class] metadata prop] {
	set pframe [keylget framelist [lindex $prop 0]]
	set klist {}
 	foreach attr [lindex $prop 1] {
	    if {[$domNode hasAttribute $attr]} {
		keylset klist $attr [$domNode getAttribute $attr]
	    } else {
		keylset klist $attr {}
	    }
 	}
	keylset klist description [tdom_getpername $domNode description]
	$pframe set_defaults $klist
    }
}
###############################################
#
# Attribute Eigenschaften
# Specializierung Editieren von Attributen 
# der Tablele (Spalten) Eigenschaften
#
###############################################
Class SchemaEditAttribute -superclass SchemaEdit
SchemaEditAttribute metadata add prop
SchemaEditAttribute metadata prop {
    {primary 
	{name type length null primary_key auto_increment default}}
    {secundary
	{valuelist notvisible noteditable constrain}}
    {metadata
	{mimetype schowpref outformat importance description}}
}
SchemaEditAttribute metadata add propspec
SchemaEditAttribute metadata propspec {
    {name {{type OLabelEntry} {textwidth 20} {state disabled}}}
    {type {{type OComboBox} {valuelist 
	{
	    blob char data datetime double enum float int longblob longtext set text timestamp tinyblob tinytext varchar   
	}
    } {state disabled}}}
    {valuelist {{type OEditList} {state disabled} }}
    {length {{type OLabelControl} {state disabled} }}
    {null {{type OCheckButton} {state disabled}}}
    {primary_key {{type OCheckButton} }}
    {auto_increment {{type OCheckButton} {state disabled}}}
    {default {{type OLabelEntry} {textwidth 20} {state disabled}}}
    {notvisible {{type OCheckButton} }}
    {noteditable {{type OCheckButton} }}
    {constrain {{type OLabelEntry} {textwidth 20} }}
    {description {{type OLabelText} }}
    {mimetype {{type OLabelEntry} {textwidth 20}}}
    {schowpref {{type OLabelEntry} {textwidth 20}}}
    {outformat {{type OLabelEntry} {textwidth 20}}}
    {importance {{type ORadioButtons} {valuelist {primary secundary extra}} }}
}
#
# Liste mit Eigenschaften, die fr ein bestmtes typ gruppe sinvoll sind
# name null default type fr Alle
#
SchemaEditAttribute metadata add typespec
SchemaEditAttribute metadata typespec {
    {
	{char varchar text blob longtext longblob tinyblob tinytext mediumblob mediumtext longblob}
	{length primary_key notvisible noteditable constrain description importance mimetype showpref}
    }
    {
	{int tinyint smallint mediumint bigint decimal float double}
	{primary_key null notvisible noteditable constrain description importance outformat}
    }
    {
	{enum set}
	{notvisible noteditable description importance valuelist}
    }
    {
	{date timestamp datetime time}
	{notvisible noteditable description importance outformat}
    }
}
SchemaEditAttribute instproc map window {
    next
} {
    [$domNode nodeType]=="ELEMENT_NODE" && [$domNode nodeName]=="attr"
} {{#post}}

SchemaEditAttribute instproc handleNewNode node {
    [self] instvar domNode framelist
    if {$domNode==$node} {return}
    set domNode $node
    #puts "FrameList $framelist"
    # Suche Alle sinnvollen Elemente fr den bestimmten Attribut-Type
    set type [$domNode getAttribute type]
    foreach typelist [[self class] metadata typespec] {
	if {[lsearch [lindex $typelist 0] $type]!=-1} {
	    set proplist [lindex $typelist 1]
	    break
	}
    }
    lappend proplist name null default type
    # Wenn primary_key==1 dann auto_increament hat auch sinn
    if {[$domNode hasAttribute primary_key] &&
	[$domNode getAttribute primary_key]=="1"} {
	lappend proplist auto_increment
    }
    # Lese alle Eigenschaften
    set actprop [[self] dom2KeyedList]
    set actkey [keylkeys actprop]
    #puts "$actprop $actkey"
    # Initialisiere die Elemente in den Kategorieen (Notebook)
    foreach prop [[self class] metadata prop] {
	set pframe [keylget framelist [lindex $prop 0]]
	set klist {}
 	foreach attr [lindex $prop 1] {
	    lappend attr
	    if {[lcontain $actkey $attr]} {
		keylset klist $attr [keylget actprop $attr]
	    } else {
		# Die ungebrauchte Attribute werden hier auf {} gesetzt
		# Probleme bei Zahlenwidget (LabelControl) da ist
		# ein leeres Wert 0
		keylset klist $attr {}
	    }
 	}
	#puts "$pframe set_defaults $klist"
	$pframe set_defaults $klist
	# Ausblende die unwichtigen Attribute
	$pframe hide [minusMenge [lindex $prop 1] $proplist]
	# Zeige die wichtigen
	$pframe show [schnittMenge [lindex $prop 1] $proplist]
    }
}
# Diese Prozedur mu die attr tag in eine keyes List umsetzten

SchemaEditAttribute instproc dom2KeyedList {} {
    [self] instvar domNode
    # Zuerst die Attribute 
    foreach attr [$domNode attributes] {
	keylset klattr $attr [$domNode getAttribute $attr]
    }
    # Dann ander child Tags valuelist oder metadata
    keylset klattr valuelist [tdom_getvaluelist $domNode]
    foreach meta_node [$domNode selectNodes child::metadata] {
	foreach ch_node [$meta_node childNodes] {
	    keylset klattr [$ch_node nodeName] [$ch_node text]
	}
    }
    return $klattr
}
# Diese Prozedur soll die Eigenschaft vernderungen in den tDom
# Struktur abbilden, Es werden hier die Kontrollen durchgefgt
# und unntige Attribute und Tags entfernt
SchemaEditAttribute instproc modifyProp propchanges {
    [self] instvar domNode
    set ch_attr [keylkeys propchanges]
    set attribute_prop {name type length null primary_key auto_increment default notvisible noteditable constrain}
    set meta_data {mimetype schowpref outformat importance description}
    
    tdom_keyl2attr $domNode $propchanges {
	{primary_key 0} {auto_increment 0}
	{default {}} {null 0} {notvisible 0} {noteditable 0} 
	{constrain {}}
    } $attribute_prop
    #foreach attr [schnittMenge $ch_attr $attribute_prop] {
    #	$domNode setAttribute $attr [keylget propchanges $attr]
    #}
    set rdom_doc [$domNode ownerDocument]
    if {[lcontain ch_attr valuelist]} {
	#Alte valuelist lschen
	tdom_setvaluelist $domNode [keylget propchanges valuelist]
# 	foreach vlist_node [$domNode getElementsByTagName valuelist] {
# 	    $domNode removeChild $vlist_node
# 	}
# 	#Neue bilden
# 	set vl_node [$rdom_doc createElement valuelist]
# 	foreach item [split $name ,] {
# 	    set item [string trim item]
# 	    set item [string trim item '\"]
# 	    set i_node [$rdom_doc createElement item]
# 	    set t_node [$rdom_doc createTextNode $item]
# 	    $i_node appendChild $t_node
# 	    $vl_node appendChild $i_node
# 	}
# 	$domNode appendChild $vl_node
    }
    set metach [schnittMenge $ch_attr $meta_data]
    # Jedes Metadata wird als ein Tag abgebildet
    if {$metach!=""} {
	set meta_node [$domNode getElementsByTagName metadata]
	if {$meta_node==""} {
	    set meta_node [$rdom_doc createElement metadata]
	    $domNode appendChild $meta_node
	}
	foreach metaitem $metach {
	    set item_node [$domNode getElementsByTagName $metaitem]
	    set newtext [string trim [keylget propchanges $metaitem]]
	    if {$item_node==""} {
		set item_node [$rdom_doc createElement $metaitem]
		$meta_node appendChild $item_node
	    } else {
		if {$newtext==""} {
		    # kein Text also Node Lschen
		    $meta_node removeChild $item_node
		    break
		}
		# Alter Text Lschen
		# vorausgesetzt es ist das erste Child
		$item_node removeChild [$item_node firstChild]
	    }
	    set item_text [$rdom_doc createTextNode $newtext]
	    $item_node appendChild $item_text	    
	}
    }
    # Hat sich primary_key gendert
    if {[lcontain $ch_attr primary_key]} {
	[self] showChanges $domNode
    }
    if {[lcontain $ch_attr name]} {
	[self] newName $domNode [$domNode getAttribute name]
    }
}
###############################################
#
# Attribute Group Eigenschaften
#
###############################################
Class SchemaEditGroup -superclass SchemaEdit
SchemaEditGroup metadata add prop
SchemaEditGroup metadata prop {
    {primary 
	{name elements}}
}
SchemaEditGroup metadata add propspec
SchemaEditGroup metadata propspec {
    {name {{type OLabelEntry} {textwidth 20} }}
    {elements {{type OMultiList} {valuelist {}} }}
}
SchemaEditGroup instproc map window {
    next
} {
    [$domNode nodeType]=="ELEMENT_NODE" && [$domNode nodeName]=="attrgroup"
} {{#post}}

SchemaEditGroup instproc handleNewNode node {
    [self] instvar domNode framelist translist lgroup
    next
    set pframe [keylget framelist primary]
    # Suche welche Attribute gehren zur Gruppe
    set lgroup {}
    foreach attr_node [$domNode selectNodes child::attr] {
	set name [$attr_node getAttribute name]
	lappend lgroup $name
	set translist($name) $attr_node
    }
    # Welche Attribute gibt es ber die Gruppe
    # keine mitglieder von weiteren Gruppen
    set plist {}
    foreach attr_node [[$domNode parentNode] selectNodes child::attr] {
	set name [$attr_node getAttribute name]
	lappend plist $name
	set translist($name) $attr_node
    }
    #puts "LG $lgroup PL $plist S [addMenge $lgroup $plist]"
    ${pframe}::elements setvaluelist [lsort [addMenge $lgroup $plist]]
    $pframe set_defaults [list [list elements $lgroup]]
}
# Diese Prozedur soll die Eigenschaft vernderungen in den tDom
# Struktur abbilden, Es werden hier die Kontrollen durchgefgt
# und unntige Attribute und Tags entfernt
SchemaEditGroup instproc modifyProp propchanges {
    [self] instvar domNode lgroup translist
    set ch_attr [keylkeys propchanges]
    if {[lcontain $ch_attr name]} {
	$domNode setAttribute name [keylget propchanges name]
	[self] newName $domNode
    }
    if {[lcontain $ch_attr elements]} {
	set elements [keylget propchanges elements]
	#  Welche Attribute gehren nicht mehr zu Gruppe
	set posnode [$domNode nextSibling]
	foreach elem [minusMenge $lgroup $elements] {
	    # Nach auen transportieren
	    [self] moveElem $translist($elem) $posnode 0
	}
	foreach elem [minusMenge $elements $lgroup] {
	    [self] moveElem $translist($elem) $domNode 1
	}
    }
}
# SchemaEditGroup instproc removeElem node {
#puts "rmElem $node"
#     set parent [$node parentNode]
#     $parent removeChild $node
#     # Vor der gruppe plazieren
#     [$parent parentNode] insertBefore $node $parent
# }
# SchemaEditGroup instproc addElem node {
#puts "addElem $node"
#     [self] instvar domNode
#     [$node parentNode] removeChild $node
#     $domNode appendChild $node
# }
#
###############################################
#
# Label Eigenschaften (Tabellen Etikette)
#
###############################################
Class SchemaEditLabel -superclass SchemaEdit
SchemaEditLabel metadata add prop
SchemaEditLabel metadata prop {
    {primary 
	{elements id-relationship}}
}
SchemaEditLabel metadata add propspec
SchemaEditLabel metadata propspec {
    {elements {{type OMultiList} {valuelist {}} }}
    {id-relationship {{type OCheckButton} }}
}
SchemaEditLabel instproc map window {
    next
} {
    [$domNode nodeType]=="ELEMENT_NODE" && [$domNode nodeName]=="attrgroup"
} {{#post}}

SchemaEditLabel instproc handleNewNode node {
    [self] instvar domNode framelist
    set domNode $node
    set pframe [keylget framelist primary]
    set labellist [tdom_getvaluelist $node]
    set attributen [concat [[$node parentNode] selectNodes child::attr/attribute::name] [[$node parentNode] selectNodes child::attrgroup/descendant::attr/attribute::name]]
    ${pframe}::elements setvaluelist $attributen
    keylset defvalue elements $labellist
    if {[[$node parentNode] hasAttribute reference]} {
	$pframe show id-relationship
	if {[$node hasAttribute id-relationship]} {
	    keylset defvalue id-relationship [$node getAttribute id-relationship]
	}
    } else {
	$pframe hide id-relationship
    }
    $pframe set_defaults $defvalue

}
# Diese Prozedur soll die Eigenschaft vernderungen in den tDom
# Struktur abbilden, Es werden hier die Kontrollen durchgefgt
# und unntige Attribute und Tags entfernt
SchemaEditLabel instproc modifyProp propchanges {
    [self] instvar domNode
    tdom_keyl2attr $domNode $propchanges {{id-relationship 0}} id-relationship
    if {[lcontain [keylkeys propchanges] elements]} {
	set elements [keylget propchanges elements]
	if {$elements==""} {
	    serviceSEdit deleteElem $domNode
	} else {
	    tdom_setvaluelist $domNode $elements
	}
    }
}
# Editieren von assoziationtarget Nodes
# Partielle Assoziation Editierung
Class SchemaEditTarget -superclass SchemaEdit
SchemaEditTarget metadata add prop
SchemaEditTarget metadata prop {
    {primary 
	{name rolename minOccurs maxOccurs ondelete onupdate}}
}
SchemaEditTarget metadata add propspec
SchemaEditTarget metadata propspec {
    {name {{type OLabelEntry} {textwidth 20} }}
    {rolename {{type OLabelEntry} {textwidth 20} }}
    {maxOccurs {{type OLabelEntry} {textwidth 20}}}
    {minOccurs {{type OLabelEntry} {textwidth 20}}}
    {ondelete {{type OComboBox} {valuelist {nothing delete setnull}} {default nothing}}}
    {onupdate {{type OComboBox} {valuelist {nothing update setnull}} {default nothing}}}
}
SchemaEditTarget instproc map window {
    next
}
SchemaEditTarget instproc handleNewNode node {
    [self] instvar domNode framelist
    set domNode $node
    set pframe [keylget framelist primary]
    set defval [tdom_attr2keyl $node {name rolename minOccurs maxOccurs ondelete onupdate}]
    tdom_setdefaults defval {
	{minOccurs {}} {maxOccurs {}} {ondelete nothing} {onupdate nothing}
	{rolename {}}
    }
    $pframe set_defaults $defval

}
SchemaEditTarget instproc modifyProp propchanges {
    [self] instvar domNode
    global lan
    tdom_keyl2attr $domNode $propchanges {
	{minOccurs {}} {maxOccurs {}} {ondelete nothing} {onupdate nothing}
    } {minOccurs maxOccurs ondelete onupdate}
    if {[lcontain [keylkeys propchanges] name]} {
	set name [keylget propchanges name]
	if {[$domNode selectNodes /descendant::assoziationtarget\[@name=\"$name\"\]]!=""} {
	    tk_messageBox -message $lan(dialog.samename) -type ok -icon error
	    return
	}
	set oldname [$domNode getAttribute name]
	foreach node [$domNode selectNodes /descendant::assoziationtarget\[@name=\"$oldname\"\]] {
	    [self] newName $node $name
	}
	# Suche assoztioncontainer. Es sollte eigentlich nur einen geben
	foreach node [$domNode selectNodes /descendant::assoziationcollection\[@name=\"$oldname\"\]] {
	    [self] newName $node $name
	}
    }
    # nderung der Rolenname mu auf zwei Stellen geschehen
    # Target und Container->Participant
    if {[lcontain [keylkeys propchanges] rolename]} {
	set rolename [keylget propchanges rolename]
	# finde die Collection
	set ac_node [$domNode selectNodes /descendant::assoziationcollection\[@name=\"[$domNode getAttribute name]\"\]]
	# existiet dieses rolename
	if {[$ac_node selectNodes child::participant\[@rolename=\"$rolename\"\]]!=""} {
	    tk_messageBox -message $lan(dialog.samename) -type ok -icon error
	    return
	}
	if {[$domNode hasAttribute rolename]} {
	    set p_node [$ac_node selectNodes child::participant\[@rolename=\"[$domNode getAttribute rolename]\"\]]
	} else {
	    set table [lindex [$domNode selectNodes ancestor::table] 0]
	    set p_node [$ac_node selectNodes child::participant\[@table=\"[$table getAttribute name]\"\]]
	}
	if {$rolename!=""} {
	    $p_node setAttribute rolename $rolename
	    $domNode setAttribute rolename $rolename
	} else {
	    $p_node removeAttribute rolename
	    $p_node removeAttribute rolename
	}
    }
}
Class SchemaEditCollection -superclass SchemaEdit
SchemaEditCollection metadata add prop
SchemaEditCollection metadata prop {
    {primary 
	{name minOccurs}}
}
SchemaEditCollection metadata add propspec
SchemaEditCollection metadata propspec {
    {name {{type OLabelEntry} {textwidth 20} }}
    {minOccurs {{type ORadioButtons} {valuelist {0 1}} }}
}
SchemaEditCollection instproc map window {
    next
}

SchemaEditCollection instproc handleNewNode node {
    [self] instvar domNode framelist
    set domNode $node
    set pframe [keylget framelist primary]
    set defval [tdom_attr2keyl $node {name minOccurs}]
    tdom_setdefaults defval {
	{minOccurs 0}
    }
    $pframe set_defaults $defval
    if {[$node getAttribute type]=="reference"} {
	$pframe show minOccurs
    } else {
	$pframe hide minOccurs
    }

}
SchemaEditCollection instproc modifyProp propchanges {
    [self] instvar domNode
    global lan
    tdom_keyl2attr $domNode $propchanges {
	{minOccurs 0}
    } {minOccurs}
    if {[lcontain [keylkeys propchanges] name]} {
	set name [keylget propchanges name]
	if {[$domNode selectNodes /descendant::assoziationtarget\[@name=\"$name\"\]]!=""} {
	    tk_messageBox -message $lan(dialog.samename) -type ok -icon error
	    return
	}
	set oldname [$domNode getAttribute name]
	foreach node [$domNode selectNodes /descendant::assoziationtarget\[@name=\"$oldname\"\]] {
	    [self] newName $node $name
	}
	# Suche assoztioncontainer. Es sollte eigentlich nur einen geben
	foreach node [$domNode selectNodes /descendant::assoziationcollection\[@name=\"$oldname\"\]] {
	    [self] newName $node $name
	}
    }
}
# Dies erweitert die Klassen FrameConteiner
# um zwei Prozeduren fr Anzeigen und Verstecken von Elementen
# Vor einen show Prozedur mu wenigstens einmal ein hide Aufruf erfolgen
#
# Eine Hilfsprozedur
proc give_pos_arg {primary_order ist_order elem} {
    #puts "PO $primary_order IS $ist_order ELEM $elem"
    set pos [lsearch $primary_order $elem]
    set len [llength $primary_order]
    # Idee Suche einen nchsten alten Nachbarn der in der ist_order ist und
    # richte sich nach ihm
    # Zuerst die Vorgnger
    for {set prev_elem [expr $pos - 1]} {$prev_elem>=0} {incr prev_elem -1} {
	set nelem [lindex $primary_order $prev_elem]
	if {[lcontain $ist_order $nelem]} {
	    return "-after $nelem"
	}
    }
    for {set next_elem [expr $pos + 1]} {$next_elem<$len} {incr next_elem} {
	set nelem [lindex $primary_order $next_elem]
	if {[lcontain $ist_order $nelem]} {
	    return "-before $nelem"
	}
    }
    return
}
FrameContainer instproc show list {
    [self] instvar win winorder
    #puts "SHOW $list"
    # Festellen welche Elemente es gibt aber die nicht angezeigt werden
    set swin [pack slaves $win]
    foreach obj $list {
	set objw [[self]::$obj set win]
	if {![lcontain $swin $objw]} {
	    # Ursprungliche Anordnung herstellen
	    set position_arg [give_pos_arg $winorder [pack slaves $win] $objw]
	    eval pack $objw [[self]::${obj}::packoption getpackoption] $position_arg
	}
    }
}
FrameContainer instproc hide list {
    [self] instvar win winorder
    #puts "HIDE $list"
    # Feststellen welche Elemente Angezeigt werden
    if {![info exist winorder]} {
	set winorder [pack slaves $win]
    }
    set swin [pack slaves $win]
    set forgetlist {}
    foreach obj $list {
	set objw [[self]::$obj set win]
	if {[lcontain $swin $objw]} {
	    lappend forgetlist $objw
	}
    }
    if {$forgetlist!=""} { eval pack forget $forgetlist }
}
